#include "structs.h"
/*!
 @file resource_bcond.h

 @brief This file contains the declaration of the resource of functions  to define boundary conditions. */

/* Functions defined in the file "bcond.c": */
PetscErrorCode CreateBcond(BcondType bCondType, PetscInt ineighb,
		Ptr_Patch Patch, Ptr_Bcond *bcond);
PetscErrorCode AddBcondApplication(Ptr_Bcond bcond, PetscInt ieq, PetscInt ivar,
		ktypeProp ktypeCoeffA, PetscReal ConsCoeffA,
		PetscErrorCode(*FunctCoeffA)(Ptr_Prop Prop), ktypeProp ktypeCoeffB,
		PetscReal ConsCoeffB, PetscErrorCode(*FunctCoeffB)(Ptr_Prop Prop));

/* WEST INLET */
PetscErrorCode BcondResourceInletWest(Ptr_Patch Patch);
PetscErrorCode Bcond_Inlet_West_UB_GeomFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_Inlet_West_VB_GeomFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_Inlet_West_UPnodeP_GeomFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_Inlet_West_PB_GeomFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_Inlet_West_KEB_GeomFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_Inlet_West_EPB_GeomFunct(Ptr_Prop Prop);

/* WEST INLET: Fix value */
PetscErrorCode BcondResourceInletFixvalWest(Ptr_Patch Patch);
PetscErrorCode Bcond_InletFixval_West_UB_GeomFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_InletFixval_West_VB_GeomFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_InletFixval_West_PB_VariFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_Inlet_West_KEB_GeomFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_Inlet_West_EPB_GeomFunct(Ptr_Prop Prop);

/* EAST OUTFLOW */
PetscErrorCode BcondResourceOutflowEast(Ptr_Patch Patch);
PetscErrorCode Bcond_Outflow_East_UB_ArrayFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_Outflow_East_VB_ArrayFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_Outflow_East_PB_ArrayFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_Outflow_East_KEB_ArrayFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_Outflow_East_EPB_ArrayFunct(Ptr_Prop Prop);

/*OUTLET: Newman*/
PetscErrorCode BcondResourceOutpressNewmanEast(Ptr_Patch Patch);
PetscErrorCode Bcond_OutpressNewman_East_UUnodeP_ArrayFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_OutpressNewman_East_UUnodeW_ArrayFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_OutpressNewman_East_UPnodeP_GeomFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_OutpressNewman_East_UPnodeW_GeomFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_OutpressNewman_East_PUnodeP_GeomFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_OutpressNewman_East_PUnodeW_GeomFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_OutpressNewman_East_KEKEnodeP_ArrayFunct(Ptr_Prop Prop);

PetscErrorCode BcondResourceOutpressNewmanPvariEast(Ptr_Patch Patch);
PetscErrorCode Bcond_OutpressNewman_East_PB_ArrayFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_OutpressNewman_East_UB_ArrayFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_OutpressNewmanPvari_East_UB_ArrayFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_OutpressNewmanPvari_East_VB_ArrayFunct(Ptr_Prop Prop);

PetscErrorCode BcondResourceOutconvEast(Ptr_Patch Patch);
PetscErrorCode Bcond_Outconv_East_UB_ArrayFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_Outconv_East_VB_ArrayFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_Outconv_East_PB_ArrayFunct(Ptr_Prop Prop);

/*OUTLET: Extrapolation*/
PetscErrorCode BcondResourceOutpressExtrapEast(Ptr_Patch Patch);
PetscErrorCode Bcond_OutpressExtrap_East_UUnodeP_ArrayFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_OutpressExtrap_East_UUnodeW_ArrayFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_OutpressExtrap_East_UPnodeP_GeomFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_OutpressExtrap_East_UPnodeW_GeomFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_OutpressExtrap_East_PUnodeP_GeomFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_OutpressExtrap_East_PUnodeW_GeomFunct(Ptr_Prop Prop);

PetscErrorCode BcondResourceOutpressExtrapNorth(Ptr_Patch Patch);
PetscErrorCode Bcond_OutpressExtrap_North_UUnodeP_ArrayFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_OutpressExtrap_North_UUnodeS_ArrayFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_OutpressExtrap_North_VPnodeP_GeomFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_OutpressExtrap_North_VPnodeS_GeomFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_OutpressExtrap_North_PVnodeP_GeomFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_OutpressExtrap_North_PVnodeS_GeomFunct(Ptr_Prop Prop);

PetscErrorCode BcondResourceOutpressExtrapSouth(Ptr_Patch Patch);
PetscErrorCode Bcond_OutpressExtrap_South_UUnodeP_ArrayFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_OutpressExtrap_South_UUnodeN_ArrayFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_OutpressExtrap_South_VPnodeP_GeomFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_OutpressExtrap_South_VPnodeN_GeomFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_OutpressExtrap_South_PVnodeP_GeomFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_OutpressExtrap_South_PVnodeN_GeomFunct(Ptr_Prop Prop);

PetscErrorCode Bcond_OutpressExtrap_East_EPEPnodeP_ArrayFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_OutpressExtrap_East_EPEPnodeW_ArrayFunct(Ptr_Prop Prop);

/* all variables, including MI terms */
PetscErrorCode Bcond_OutpressExtrapWhole_East_UUnodeP_ArrayFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_OutpressExtrapWhole_East_UUnodeW_ArrayFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_OutpressExtrapWhole_East_PPnodeP_ArrayFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_OutpressExtrapWhole_East_PPnodeW_ArrayFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_OutpressExtrapWhole_East_PB_ArrayFunct(Ptr_Prop Prop);

/* WALLS */
PetscErrorCode BcondResourceWallNorth(Ptr_Patch Patch);
PetscErrorCode BcondResourceWallSouth(Ptr_Patch Patch);
PetscErrorCode BcondResourceWallEast(Ptr_Patch Patch);
PetscErrorCode BcondResourceWallWest(Ptr_Patch Patch);
PetscErrorCode Bcond_Wall_XDIR_UUnodeP_GeomFunct(Ptr_Prop);
PetscErrorCode Bcond_Wall_YDIR_VVnodeP_GeomFunct(Ptr_Prop);
PetscErrorCode Bcond_Wall_North_VPnodeP_GeomFunct(Ptr_Prop);
PetscErrorCode Bcond_Wall_South_VPnodeP_GeomFunct(Ptr_Prop);
PetscErrorCode Bcond_Wall_East_UPnodeP_GeomFunct(Ptr_Prop);
PetscErrorCode Bcond_Wall_West_UPnodeP_GeomFunct(Ptr_Prop);
PetscErrorCode Bcond_Wall_XDIR_EPEPnodeP_GeomFunct(Ptr_Prop);
PetscErrorCode Bcond_Wall_YDIR_EPEPnodeP_GeomFunct(Ptr_Prop);
PetscErrorCode Bcond_Wall_XDIR_UUnodeP_GeomFunct(Ptr_Prop);
PetscErrorCode Bcond_Wall_North_PB_ArrayFunct(PetscInt I[3], Ptr_Patch Patch, PetscReal *Array[]);
PetscErrorCode Bcond_Wall_South_PB_ArrayFunct(PetscInt I[3], Ptr_Patch Patch, PetscReal *Array[]);
PetscErrorCode Bcond_Wall_West_PB_ArrayFunct(PetscInt I[3], Ptr_Patch Patch, PetscReal *Array[]);
PetscErrorCode Bcond_Wall_East_PB_ArrayFunct(PetscInt I[3], Ptr_Patch Patch, PetscReal *Array[]);

PetscErrorCode Bcond_Wall_East_UPnodeW_GeomFunct(PetscInt I[3], Ptr_Patch Patch,
		PetscReal *Array[]);
PetscErrorCode Bcond_Wall_West_UPnodeE_GeomFunct(PetscInt I[3], Ptr_Patch Patch,
		PetscReal *Array[]);
/* MOVING WALL */
PetscErrorCode BcondResourceMovingWallNorth(Ptr_Patch Patch);
PetscErrorCode Bcond_Wall_North_UB_GeomFunct(Ptr_Prop);

/* LAW WALL */
PetscErrorCode BcondResourceWallLawNorth(Ptr_Patch Patch);
PetscErrorCode BcondResourceWallLawSouth(Ptr_Patch Patch);
PetscErrorCode BcondResourceWallLawEast(Ptr_Patch Patch);
PetscErrorCode BcondResourceWallLawWest(Ptr_Patch Patch);
PetscErrorCode Bcond_WallLaw_North_UUnodeP_Funct(Ptr_Prop Prop);
PetscErrorCode Bcond_WallLaw_North_VVnodeP_Funct(Ptr_Prop Prop);
PetscErrorCode Bcond_WallLaw_South_UUnodeP_Funct(Ptr_Prop Prop);
PetscErrorCode Bcond_WallLaw_South_VVnodeP_Funct(Ptr_Prop Prop);
PetscErrorCode Bcond_WallLaw_East_UUnodeP_Funct(Ptr_Prop Prop);
PetscErrorCode Bcond_WallLaw_East_VVnodeP_Funct(Ptr_Prop Prop);
PetscErrorCode Bcond_WallLaw_West_UUnodeP_Funct(Ptr_Prop Prop);
PetscErrorCode Bcond_WallLaw_West_VVnodeP_Funct(Ptr_Prop Prop);
PetscErrorCode Bcond_WallLaw_XDIR_VVnodeP_GeomFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_WallLaw_YDIR_VVnodeP_GeomFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_WallLaw_YDIR_UUnodeP_GeomFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_WallLaw_XDIR_KEnodeP_Funct(Ptr_Prop Prop);
PetscErrorCode Bcond_WallLaw_XDIR_EPnodeP_Funct(Ptr_Prop Prop);
PetscErrorCode Bcond_WallLaw_TwA_GeomFunct(PetscInt I[3], PetscInt vari,
    PetscInt ineighb, PetscReal* value);
PetscErrorCode Bcond_UT_wall_Funct(PetscReal rho, PetscReal visc, PetscReal y,
		PetscReal vel,PetscReal *value);

/* CORNERS */
PetscErrorCode BcondResourceCornerWestNorth(Ptr_Patch Patch);
PetscErrorCode BcondResourceCornerWestSouth(Ptr_Patch Patch);
PetscErrorCode BcondResourceCornerEastSouth(Ptr_Patch Patch);
PetscErrorCode BcondResourceCornerEastNorth(Ptr_Patch Patch);
PetscErrorCode Bcond_Corner_WestSouth_PB_GeomFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_Corner_WestNorth_PB_GeomFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_Corner_EastSouth_PB_GeomFunct(Ptr_Prop Prop);
PetscErrorCode Bcond_Corner_EastNorth_PB_GeomFunct(Ptr_Prop Prop);

/* SYMMETRY AXIS */
PetscErrorCode BcondResourceSymmetryNorth(Ptr_Patch Patch);
PetscErrorCode BcondResourceSymmetrySouth(Ptr_Patch Patch);
PetscErrorCode Bcond_Symmetry_South_VPnodeP_GeomFunct(Ptr_Prop);
PetscErrorCode Bcond_Symmetry_North_VPnodeP_GeomFunct(Ptr_Prop);
PetscErrorCode Bcond_Symmetry_South_VB_VariFunct(Ptr_Prop);
PetscErrorCode Bcond_Symmetry_North_VB_VariFunct(Ptr_Prop);
PetscErrorCode Bcond_Symmetry_North_UUnodeP_ArrayFunct(Ptr_Prop);
PetscErrorCode Bcond_Symmetry_North_PVnodeP_ArrayFunct(Ptr_Prop);
PetscErrorCode Bcond_Symmetry_South_UUnodeP_ArrayFunct(Ptr_Prop);
PetscErrorCode Bcond_Symmetry_South_PVnodeP_ArrayFunct(Ptr_Prop);

/* FIX ZERO VALUE */
PetscErrorCode BcondFixZeroValues(Ptr_Patch Patch);

/* PRESSURE SOURCE */
PetscErrorCode Bcond_PressureSource_UB_GeomFunct(Ptr_Prop Prop);

/* INLET FUNCTIONS */
PetscErrorCode UInletFunct(PetscInt*, PetscReal*);
PetscErrorCode FInletFunct(PetscInt*, PetscReal*);
PetscErrorCode RHOInletFunct(PetscInt*, PetscReal*);

