#include "structs.h"

extern St_Global Global; /*!< Global properties of the problem. */
extern St_Local Local; /*!< Local properties of the problem. */

extern St_Input Input; /*!< Input data. */
extern St_Grid Grid; /*!< Global grid definition and parameters. */
extern St_Geom Geom; /*!< Geometrical properties derived from the grid. */
extern St_Trans Trans; /*!< Parameters for solving unsteady problems. */
extern Ptr_Prop Rho, ViscTKE, ViscTLES;
extern Ptr_Patch DomainPatch;
extern Ptr_Var varU, varV, varW, varP, varT, varKE, varEP;

PetscErrorCode MICScInitialize(PetscInt *argc, char **argv[]);
PetscErrorCode MICScSolve();
PetscErrorCode MICScFinalize();

PetscErrorCode MICScVarSetLimits(int index, PetscReal minValue,
		PetscReal maxValue);
PetscErrorCode MICScVarSetRelaxation(int index, PetscReal linRelax,
		PetscReal fdtRelax);
PetscErrorCode MICScVarSetReference(int index, PetscReal resRef,
		PetscReal corrRef);
PetscErrorCode MICScVarSetInterpolation(int index, ktypeIp interpType);
PetscErrorCode MICScVarSetFunctions(int index, ktypeProp initValuesType,
    StorageType initStorage, ktypeIp initValuesInterpType, PetscReal
    initValuesConst, PetscErrorCode(*initValuesFunct)(Ptr_Prop prop),
    ktypeProp gammaType, StorageType gammaStorage, ktypeIp gammaInterpType,
    PetscReal gammaConst, PetscErrorCode(*gammaFunct)(Ptr_Prop prop));
PetscErrorCode MICScVarSetInitValues(int index, ktypeProp initValuesType,
		StorageType storage, ktypeIp initValuesInterpType, PetscReal
		initValuesConst, PetscErrorCode(*initValuesFunct)(Ptr_Prop prop));
PetscErrorCode MICScVarSetGamma(int index, ktypeProp gammaType,
    StorageType storage, ktypeIp gammaInterpType, PetscReal gammaConst,
		PetscErrorCode(*gammaFunct)(Ptr_Prop prop));

PetscErrorCode MICScSetProp(Ptr_Prop prop, Ptr_Patch patch, ktypeProp ktype,
    StorageType storage, PetscReal Cons, PetscErrorCode(*Funct)(Ptr_Prop Prop),
    ktypeIp kip, PetscReal linrlx, PetscTruth calcMeans);
PetscErrorCode MICScSetInletFunction(PetscErrorCode (*function)(PetscInt*,
		PetscReal*));
